#brailleDisplayDrivers/erbnvda.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012 NVDA Contributors <http://www.nvda-project.org/>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

import braille
from logHandler import log
from ctypes import *
#from ctypes.wintypes import *
from time import sleep
from wx import PyTimer
import config
import inputCore
#import os

KEY_CHECK_INTERVAL = 300

# EUROBRAILLE KEY TYPES
EUROBRAILLE_ROUTING_KEYS = 01
EUROBRAILLE_DOUBLE_ROUTING_KEYS = 02
EUROBRAILLE_POSITION_KEYS = 03
EUROBRAILLE_BRAILLE_KEYS = 04
EUROBRAILLE_COMMAND_KEYS = 05

# EUROBRAILLE KEYS
L1                     = 0x00000001
L2                     = 0x00000002        
L3                     = 0x00000004
L4                     = 0x00000008
L5                     = 0x00000010
L6                     = 0x00000020
L7                     = 0x00000040
L8                     = 0x00000080
J1UP           = 0x00000100
J1DOWN         = 0x00000200
J1RIGHT        = 0x00000400
J1LEFT         = 0x00000800
J1MIDDLE       = 0x00010000    
J2UP           = 0x00001000
J2DOWN         = 0x00002000
J2RIGHT        = 0x00004000
J2LEFT         = 0x00008000
J2MIDDLE       = 0x00020000

keyNames={
	L1: "L1",
	L8: "L8",
	J1UP: "J1UP",
	J1DOWN: "J1DOWN",
	J1LEFT: "J1LEFT",
	J1RIGHT: "J1RIGHT",
	J1MIDDLE: "J1MIDDLE",
	J2UP: "J2UP",
	J2DOWN: "J2DOWN",
	J2LEFT: "J2LEFT",
	J2RIGHT: "J2RIGHT",
	J2MIDDLE: "J2MIDDLE",
	J1UP+L1: 'J1UP+L1',
	J1UP+L8: 'J1UP+L8',
	J1UP+J2UP: 'J1UP+J2UP',
	J1UP+J2DOWN: 'J1UP+J2DOWN',
	J1UP+J2LEFT: 'J1UP+J2LEFT',
	J1UP+J2RIGHT: 'J1UP+J2RIGHT',
	J1UP+J2MIDDLE: 'J1UP+J2MIDDLE',
	J1DOWN+L1: 'J1DOWN+L1',
	J1DOWN+L8: 'J1DOWN+L8',
	J1DOWN+J2UP: 'J1DOWN+J2UP',
	J1DOWN+J2DOWN: 'J1DOWN+J2DOWN',
	J1DOWN+J2LEFT: 'J1DOWN+J2LEFT',
	J1DOWN+J2RIGHT: 'J1DOWN+J2RIGHT',
	J1DOWN+J2MIDDLE: 'J1DOWN+J2MIDDLE',
	J1LEFT+L1: 'J1LEFT+L1',
	J1LEFT+L8: 'J1LEFT+L8',
	J1LEFT+J2UP: 'J1LEFT+J2UP',
	J1LEFT+J2DOWN: 'J1LEFT+J2DOWN',
	J1LEFT+J2LEFT: 'J1LEFT+J2LEFT',
	J1LEFT+J2RIGHT: 'J1LEFT+J2RIGHT',
	J1LEFT+J2MIDDLE: 'J1LEFT+J2MIDDLE',
	J1RIGHT+L1: 'J1RIGHT+L1',
	J1RIGHT+L8: 'J1RIGHT+L8',
	J1RIGHT+J2UP: 'J1RIGHT+J2UP',
	J1RIGHT+J2DOWN: 'J1RIGHT+J2DOWN',
	J1RIGHT+J2LEFT: 'J1RIGHT+J2LEFT',
	J1RIGHT+J2RIGHT: 'J1RIGHT+J2RIGHT',
	J1RIGHT+J2MIDDLE: 'J1RIGHT+J2MIDDLE',
	J2UP+L1: 'J2UP+L1',
	J2UP+L8: 'J2UP+L8',
	J2DOWN+L1: 'J2DOWN+L1',
	J2DOWN+L8: 'J2DOWN+L8',
	J2LEFT+L1: 'J2LEFT+L1',
	J2LEFT+L8: 'J2LEFT+L8',
	J2RIGHT+L1: 'J2RIGHT+L1',
	J2RIGHT+L8: 'J2RIGHT+L8',
}

#KEY_NAMES = {
#	EUROBRAILLE_ROUTING_KEYS: None,
#	EUROBRAILLE_POSITION_KEYS: None,
#	EUROBRAILLE_BRAILLE_KEYS:	 ("b9", "bA", # byte 2
#								"b1", "b2", "b3", "b4", "b5", "b6", "b7", "b8"), # byte 1
#	EUROBRAILLE_JOYSTICK1_KEYS: ("up", "left", "down", "right", "center"),
#	EUROBRAILLE_JOYSTICK2_KEYS: ("up", "left", "down", "right", "center"),
#	EUROBRAILLE_COMMAND_KEYS: ("l1", "l2", "l3", "l4", "l5", "l6", "l7", "l8"),
#}

#Try to load erbnvda.dll
try:
	path = config.getUserDefaultConfigPath() + "\\addons\\eurobrailleDriver\\brailleDisplayDrivers"
	path = path.encode("mbcs")
	log.info("getUserDefaultConfigPath is %s" %path)
#	currentDir = os.getcwdu()
#	log.info("currentDir %s" %currentDir)

	msvcp120Lib=CDLL(path + "\\msvcr120.dll")
	log.info("erbnvda-msvcr120 loaded")
	msvcr120=CDLL(path + "\\msvcp120.dll")
	log.info("erbnvda-msvcp120 loaded")
	icudt52=CDLL(path + "\\icudt52.dll")
	log.info("erbnvda-icudt52 loaded")
	icuuc52=CDLL(path + "\\icuuc52.dll")
	log.info("erbnvda-icuuc52 loaded")	
	icuin52=CDLL(path + "\\icuin52.dll")
	log.info("erbnvda-icuin52 loaded")
	qt5CoreLib=CDLL(path + "\\qt5core.dll")
	log.info("erbnvda-Qt5Core loaded")
	qt5gui=CDLL(path + "\\qt5gui.dll")
	log.info("erbnvda-Qt5Gui loaded")
	Qt5Widgets=CDLL(path + "\\Qt5Widgets.dll")
	log.info("erbnvda-Qt5Widgets loaded")
	EurobrailleLib=CDLL(path + "\\erbnvda.dll")
	log.info("erbnvda-Eurobraille loaded")
except:
	EurobrailleLib=None
	log.info("erbnvda-Eurobraille load failed")

class BrailleDisplayDriver(braille.BrailleDisplayDriver):
	name = "erbnvda"
	# Translators: The name of a braille display.
	description = _("Eurobraille driver")

	@classmethod
	def check(cls):
		return bool(EurobrailleLib)

	def __init__(self):
		super(BrailleDisplayDriver,self).__init__()
		path = config.getUserDefaultConfigPath() + "\\addons\\eurobrailleDriver\\brailleDisplayDrivers"
		#log.info("erbnvda-EurobrailleOpen path %s" %path)
		result = EurobrailleLib.EurobrailleOpen(path)
		#log.info("erbnvda-EurobrailleOpen return %d" %result)
		self._eurobraille_NumCells = 0
		self._key = {}

		# Arm timer to keyboard survey
		self._keyCheckTimer = PyTimer(self._handleKeyPresses)
		self._keyCheckTimer.Start(KEY_CHECK_INTERVAL)

	def terminate(self):
		super(BrailleDisplayDriver, self).terminate()
		# Exceptions might be raised if initialisation failed. Just ignore them.
		try:
			self._keyCheckTimer.Stop()
			self._keyCheckTimer = None
		except:
			pass
		log.info("erbnvda-Eurobraille driver terminate")
		result = EurobrailleLib.EurobrailleClose()

	def _get_numCells(self):
		if self._eurobraille_NumCells==0:
			NumCells = c_int(0)
			EurobrailleLib.EurobrailleNumCells(byref(NumCells))
#			if NumCells.value==0: #Ignore: if numCells=0, Braille will be disabled by NVDA
#				raise RuntimeError("Cannot obtain number of cells")
			self._eurobraille_NumCells = NumCells.value
			log.info("erbnvda-Eurobraille device has %d cells" %self._eurobraille_NumCells)
		return self._eurobraille_NumCells

	def display(self, cells):
#		NumCells = 0 means that no terminal is connected
		if self._eurobraille_NumCells == 0: return
		cells="".join([chr(x) for x in cells])
		#work around a bug which causes terminal freezing 
		self._handleKeyPresses()
		sleep(0.03)
		EurobrailleLib.EurobrailleSendBraille(cells, len(cells))
		#Trace braille message.
		#log.info("erbnvda-Cells length:%d", len(cells)) 
		#log.info("erbnvda-Cells string [%s]", cells.encode("hex"));

	def _handleKeyPresses(self):
		keyType = c_long(0)
		keyValue = c_long(0)
		EurobrailleLib.EurobrailleGetKey(byref(keyType), byref(keyValue))
		if keyType.value != 0:
			self._key["type"] = keyType.value
			self._key["value"] = keyValue.value
			self._onKeyPress(self._key)
			
	def _onKeyPress(self, key):
#		log.info("erbnvda-Eurobraille _onKeyPress")
#		log.info("Type:" + str(key["type"]))
#		log.info("Value:" + str(key["value"]))
		command = key["type"]
		argument = key["value"]
		if command == EUROBRAILLE_ROUTING_KEYS:
			try:
				inputCore.manager.executeGesture(InputGesture("routing", argument))
			except inputCore.NoInputGestureAction:
				pass
		elif command == EUROBRAILLE_COMMAND_KEYS:
			#log.info("erbnvda-Eurobraille command key %d" %command)
			try:
				key = argument
				#log.info("---------------------- command %s" %key)
 				cmdStr = keyNames[argument]
				log.info("---------------------- command %s" %cmdStr)
				inputCore.manager.executeGesture(InputGesture(cmdStr, argument))
			except inputCore.NoInputGestureAction:
				pass
		else:
			log.info("erbnvda-Eurobraille unknown command %d" %command)
			

	gestureMap = inputCore.GlobalGestureMap({
		"globalCommands.GlobalCommands": {
			"braille_scrollBack": ("br(erbnvda):L1",),
			"braille_scrollForward": ("br(erbnvda):L8",),
			"kb:windows": ("br(erbnvda):J1LEFT",),
			"kb:escape": ("br(erbnvda):J1UP",),
			"kb:applications": ("br(erbnvda):J1RIGHT",),
			"kb:leftArrow": ("br(erbnvda):J2LEFT",),
			"kb:rightArrow": ("br(erbnvda):J2RIGHT",),
			"kb:upArrow": ("br(erbnvda):J2UP",),
			"kb:downArrow": ("br(erbnvda):J2DOWN",),
			"kb:tab": ("br(erbnvda):J1DOWN+J2DOWN",),
			"kb:shift+tab": ("br(erbnvda):J1DOWN+J2UP",),
			"kb:alt+shift+tab": ("br(erbnvda):J1RIGHT+L1",),
			"kb:alt+tab": ("br(erbnvda):J1RIGHT+L8",),
			"braille_previousLine": ("br(erbnvda):J1LEFT+L1",),
			"braille_nextLine": ("br(erbnvda):J1LEFT+L8",),
			"kb:control+home": ("br(erbnvda):J1UP+L1",),
			"kb:control+end": ("br(erbnvda):J1UP+L8",),
			"rightMouseClick": ("br(erbnvda):J1RIGHT+J2RIGHT",),
			"leftMouseClick": ("br(erbnvda):J1RIGHT+J2LEFT",),
			"moveMouseToNavigatorObject": ("br(erbnvda):J1RIGHT+J2UP",),
			"toggleLeftMouseButton": ("br(erbnvda):J1RIGHT+J2MIDDLE",),
			"braille_toggleTether": ("br(erbnvda):J1LEFT+J2DOWN",),
			"braille_toggleShowCursor": ("br(erbnvda):J1LEFT+J2LEFT",),
			"braille_cycleCursorShape": ("br(erbnvda):J1LEFT+J2RIGHT",),
			"toggleInputHelp": ("br(erbnvda):J1LEFT+J2UP",),
			"braille_routeTo": ("br(erbnvda):routing",),
		},
		"virtualBuffers.VirtualBuffer": {
			"elementsList": ("br(erbnvda):J1UP+J2UP",),
		},
		"globalPlugins.NVDAExtensionGlobalPlugin.GlobalPlugin": {
			"report_SystrayIconsOrWindowsList": ("br(erbnvda):J1RIGHT+J2DOWN",),
		}
	})

class InputGesture(braille.BrailleDisplayGesture):

	source = BrailleDisplayDriver.name

	def __init__(self, command, argument):
		super(InputGesture, self).__init__()
#		log.info("erbnvda-Eurobraille __init__ InputGesture %s" %command)
#		log.info( "- %d" %argument)
		self.keyNames = names = set()
		names.add(command)
		self.routingIndex = argument
		self.id = "+".join(names)
