#erbnva_globalPlugin/dialogs.py
#A part of the eurobraille driver for NVDA (Non Visual Desktop Access)
#Copyright (C) 2012 Didier Poitou <d.poitou@eurobraille.fr>
#Copyright (C) 2012 Eurobraille sas <www.eurobraille.com>

import gettext
import addonHandler
addonHandler.initTranslation()
import core
import os
import wx
import gui
import config
from configobj import ConfigObj
from logHandler import log


class ErbNvdaSettingsDialog(gui.SettingsDialog):
	title = _("Eurobraille driver settings")
	def __init__(self, parent):
		self.path = config.getUserDefaultConfigPath() + "\\addons\\eurobrailleDriver\\brailleDisplayDrivers"
		# Create parameters list of combobox.
		self._port = ["usb", "comx", "com1", "com2", "com3", "com4", "com5", "com6", "com7", "com8", "com9"]
		self._keyboard = ["Generic", "USB-HID"]
		brailleTables = os.listdir(self.path + "\\BrailleTables")
		self._charactersTables = filter(lambda file: file.endswith('.kbr'), brailleTables)
		self._functionsTables = filter(lambda file: file.endswith('.fbr'), brailleTables)
			
		# Reading .ini file.
		iniConfig = ConfigObj(self.path + "\\erbnvda.ini")
		section = iniConfig['General']
		self.port = section['CommunicationPort']
		self.keyboard = section['BrailleKeyboardMode']
		self.charactersTable = section['BrailleKeyboardTable']
		self.functionsTable = section['BrailleFunctionTable']
		self.autoForward = section['AutoForward']
		self.brailleMoving = section['BrailleMoving']
		self.openClose = section['OpenClose']
		#log.info("erbnvda-configuration port %s" %self.port)
		#log.info("erbnvda-configuration keyboard %s" %self.keyboard)
		#log.info("erbnvda-configuration port %s" %self.charactersTable)
		#log.info("erbnvda-configuration port %s" %self.functionsTable)
		#log.info("erbnvda-configuration port %s" %self.autoForward)
		
		super(ErbNvdaSettingsDialog, self).__init__(parent)

	def makeSettings(self, sizer):
		# Create port selector.
		portsSizer = wx.BoxSizer(wx.HORIZONTAL)
		portsLabel = wx.StaticText(self, label=_("Port:"))
		portsSizer.Add(portsLabel)
		portListID=wx.NewId()
		self._portsChoice=wx.Choice(self,portListID,name=_("Port"),choices=self._port)
		self._portsChoice.SetToolTip(wx.ToolTip(_("Choose the port used by braille device.")))
		portsSizer.Add(self._portsChoice,border=10,flag=wx.BOTTOM)
		index = 0
		for n, port in enumerate(self._port) :
			if self.port == port:
				index = n
		self._portsChoice.SetSelection(index)
		sizer.Add(portsSizer)
		
		# Create keyboard selector. 
		keyboardsSizer = wx.BoxSizer(wx.HORIZONTAL)
		keyboardsLabel = wx.StaticText(self, label=_("Keyboard:"))
		keyboardsSizer.Add(keyboardsLabel)
		keyboardListID=wx.NewId()
		self._keyboardsChoice=wx.Choice(self,portListID,name=_("Keyboard mode"),choices=self._keyboard)
		self._keyboardsChoice.SetToolTip(wx.ToolTip(_("Choose the keyboard mode.")))
		self.Bind(wx.EVT_CHOICE, self.onKeyboardChanged, self._keyboardsChoice)
		keyboardsSizer.Add(self._keyboardsChoice,border=10,flag=wx.BOTTOM)
		index = 0
		for n, keyboard in enumerate(self._keyboard) :
			if self.keyboard == keyboard:
				index = n
		self._keyboardsChoice.SetSelection(index)
		sizer.Add(keyboardsSizer)

		# Create braille characters table.
		charactersTablesSizer = wx.BoxSizer(wx.HORIZONTAL)
		charactersTablesLabel = wx.StaticText(self, label=_("Characters table:"))
		charactersTablesSizer.Add(charactersTablesLabel)
		keyboardListID=wx.NewId()
		self._charactersTablesChoice=wx.Choice(self,portListID,name=_("Keyboard mode"),choices=self._charactersTables)
		self._charactersTablesChoice.SetToolTip(wx.ToolTip(_("Choose the keyboard mode.")))
		charactersTablesSizer.Add(self._charactersTablesChoice,border=10,flag=wx.BOTTOM)
		index = 0
		for n, charactersTable in enumerate(self._charactersTables) :
			if self.charactersTable == charactersTable:
				index = n
		self._charactersTablesChoice.SetSelection(index)
		sizer.Add(charactersTablesSizer)

		# Create braille functions table.		
		functionsTablesSizer = wx.BoxSizer(wx.HORIZONTAL)
		functionsTablesLabel = wx.StaticText(self, label=_("functions table:"))
		functionsTablesSizer.Add(functionsTablesLabel)
		keyboardListID=wx.NewId()
		self._functionsTablesChoice=wx.Choice(self,portListID,name=_("Keyboard mode"),choices=self._functionsTables)
		self._functionsTablesChoice.SetToolTip(wx.ToolTip(_("Choose the keyboard mode.")))
		functionsTablesSizer.Add(self._functionsTablesChoice,border=10,flag=wx.BOTTOM)
		index = 0
		for n, functionsTable in enumerate(self._functionsTables) :
			if self.functionsTable == functionsTable:
				index = n
		self._functionsTablesChoice.SetSelection(index)
		sizer.Add(functionsTablesSizer)
		
		# Create checkbox for optical bar parameters.
		autoForwardSizer = wx.BoxSizer(wx.HORIZONTAL)
		self._autoForwardCheckBox=wx.CheckBox(self, -1, _("Optical bar: Auto forward"))
		if self.autoForward == "1":
			self._autoForwardCheckBox.SetValue(True)
		autoForwardSizer.Add(self._autoForwardCheckBox,border=10,flag=wx.BOTTOM)
		sizer.Add(autoForwardSizer)
		brailleMovingSizer = wx.BoxSizer(wx.HORIZONTAL)
		self._brailleMovingCheckBox=wx.CheckBox(self, -1, _("Optical bar: Braille moving functions"))
		if self.brailleMoving == "1":
			self._brailleMovingCheckBox.SetValue(True)
		brailleMovingSizer.Add(self._brailleMovingCheckBox,border=10,flag=wx.BOTTOM)
		sizer.Add(brailleMovingSizer)
		openCloseSizer = wx.BoxSizer(wx.HORIZONTAL)
		self._openCloseCheckBox=wx.CheckBox(self, -1, _("Optical bar: Open-close functions"))
		if self.openClose == "1":
			self._openCloseCheckBox.SetValue(True)
		openCloseSizer.Add(self._openCloseCheckBox,border=10,flag=wx.BOTTOM)
		sizer.Add(openCloseSizer)
		# enable braille table according to the keyboard mode.
		self.enableBrailleTable()

	def onOk(self, event):
		super(ErbNvdaSettingsDialog, self).onOk(event)
		# Update Configuration
		newPort = self._port[self._portsChoice.GetSelection()]
		newKeyboard = self._keyboard[self._keyboardsChoice.GetSelection()]
		newCharactersTable = self._charactersTables[self._charactersTablesChoice.GetSelection()]
		newFunctionsTable = self._functionsTables[self._functionsTablesChoice.GetSelection()]
		newAutoForward = "0"
		if self._autoForwardCheckBox.GetValue() :
			newAutoForward = "1"
		newBrailleMoving = "0"
		if self._brailleMovingCheckBox.GetValue():
			newBrailleMoving = "1"
		newOpenClose = "0"
		if self._openCloseCheckBox.GetValue():
			newOpenClose = "1"
		if (newPort != self.port) or (newKeyboard != self.keyboard) or (newCharactersTable != self.charactersTable) or (newFunctionsTable != self.functionsTable) or (newAutoForward != self.autoForward) or (newBrailleMoving != self.brailleMoving) or (newOpenClose != self.openClose):
			iniConfig = ConfigObj(self.path + "\\erbnvda.ini")
			section = iniConfig['General']
			section['CommunicationPort'] = newPort
			section['BrailleKeyboardMode'] = newKeyboard
			section['BrailleKeyboardTable'] = newCharactersTable
			section['BrailleFunctionTable'] = newFunctionsTable
			section['AutoForward'] = newAutoForward
			section['BrailleMoving'] = newBrailleMoving
			section['OpenClose'] = newOpenClose
			iniConfig.write()
			
			# Translators: A message asking the user if they wish to restart NVDA as addons have been added or removed. 
			if gui.messageBox(_("Add-on has been modified. You must restart NVDA for these changes to take effect. Would you like to restart now?"),
			# Translators: Title for message asking if the user wishes to restart NVDA as addons have been added or removed. 
			_("Restart NVDA"),
			wx.YES|wx.NO|wx.ICON_WARNING)==wx.YES:
				core.restart()

		return True

	# enable braille table according to the keyboard mode.
	def enableBrailleTable(self):
		if self._keyboard[self._keyboardsChoice.GetSelection()] == "Generic":
			self._charactersTablesChoice.Enable()
			self._functionsTablesChoice.Enable()
		else:
			self._charactersTablesChoice.Disable()
			self._functionsTablesChoice.Disable()

	def onKeyboardChanged(self, event):
		#log.info("erbnvda-configuration keyboard change %s" %self._keyboard[self._keyboardsChoice.GetSelection()])
		self.enableBrailleTable()
				